Macro "Batch Maps"
    // This macro prints a batch of maps based on chosen map features.
    // It displays a list of store names and addresses from a store points
    // layer, lets you build a list of chosen stores and then, for each
    // chosen store, automatically creates a layout on your default printer
    // and adds a map scoped to the store and the area around it, along with
    // a legend and a title containing the store name.
    // A test mode is provided which displays the prepared layout without
    // actually printing it.
    // The macro currently uses the Stores layer (NYC_SPLY.CDF) in the
    // tutorial folder. You can change the macro to use your own data.

    // To run this macro using the Maptitude GISDK toolbox:
    // 1. Use the Windows menu choices Start-Settings-Printers to set the printer
    //    you want to use for printing to be the default.  Choose landscape for
    //    the orientation.
    // 2. In Maptitude, choose File-Open and open the geographic file NYC_SPLY.CDF 
    //    in the Maptitude tutorial folder.  The layer name will be "Stores". 
    // 3. Click Automatic Labels on the toolbar and label the layer
    //    with the Name field.
    // 4. VERY IMPORTANT: click Show or Hide the Legend on the toolbar to turn
    //    on the legend.
    // 5. Add other desired layers such as streets (CCSTREET.CDF) from the data 
    //    you installed with Maptitude.
    // 6. Choose Tools-Add-Ins and open the GIS Developer's Kit toolbox.
    // 7. Compile this macro in test mode using the first button (Compile).
    // 8. Run this macro by clicking the second button (Test), typing the macro 
    //    name "Batch Maps", and clicking OK. The type of add-in is macro
    //   (the default).
    // 9. Choose and arrange the stores to print in the dialog box and click OK. 

    // To adapt this macro to another points layer, change the settings in the 
    // Administrative Settings section below.

    // Administrative Settings -------------------------------------------------
    // This section contains variables that won't change while the macro is in
    // use, but make it easy to adapt the macro to a new situation. You could also
    // let the user set or change these variables in a dbox.

    // Points layer name to use for choosing locations.
    points_layer = "Stores"

    // Field name of name field in point_layer to be used for displaying a list.
    name_field = "Name"

    // Width of map in miles; map scope will be set to this width.
    map_width_miles = 1.0 

    // Position of map on layout in inches from upper left hand corner.
    // Note: using inches here makes it easier to specify locations.  Later
    // they will be converted to layout dimensions which are in hundredths of
    // points (7,200 per inch).
    map_ulx = 0.5       // upper left x position
    map_uly = 1.0       // upper left y position
    map_lrx = 8.5       // lower right x position
    map_lry = 7.5       // lower right y position

    // Upper left hand corner of legend on layout in inches from upper left
    // hand corner of layout.
    legend_ulx = 8.75   // upper left x position
    legend_uly = 1.0    // upper left y position

    // Position of title layout in inches from upper left hand corner.
    title_ulx = 3.0     // upper left x position
    title_uly = .5      // upper left y position
    title_lrx = 5.0     // lower right x position
    title_lry = .8      // lower right y position
    // -------------------------------------------------------------------------

    // Call the dbox to prompt the user, passing the layer and name field.  Also
    // pass the test variable with "&" prefix so that you can get the value the
    // user supplied.  The dbox returns the chosen points in an array.  If the
    // user pressed "Cancel" or chose no points then null is returned.
    chosen_pts = RunDbox("get_choices", points_layer, name_field, &test)

    // If nothing to do then return.
    if chosen_pts = null then return()

    // Set the map aspect ratio (width divided by height) to fit that of the
    // location rectangle specified for the map on the layout.

    // Get the current map window size.
    sz = GetWindowSize()
    map_window_wd = sz[1]  // width as percent
    map_window_ht = sz[2]  // height as percent

    // Compute aspect ratio of map on layout.
    layout_ar = (map_lrx-map_ulx)/(map_lry-map_uly)  

    // Set the window to 600 pixels wide, and adjust the height to match the
    // aspect ratio on the layout.
    SetWindowSizePixels(, 600, r2i(600/layout_ar))

    // Create location variables for use in AddAnnotation().
    // All locations on a layout are specified in hundredths of points, so you
    // must convert the inches in the Administrative Settings section to
    // hundredths of points and create location arrays containing two coordinates
    // which form a rectangle.
    SetMapUnits("Miles")
    ppi = 7200  // layout units (hundredths of points) per inch
    map_loc = {Coord(r2i(map_ulx*ppi), r2i(map_uly*ppi)),
               Coord(r2i(map_lrx*ppi), r2i(map_lry*ppi))}
    // Only need upper left corner of legend
    legend_loc = Coord(r2i(legend_ulx*ppi), r2i(legend_uly*ppi))
    title_loc = {Coord(r2i(title_ulx*ppi), r2i(title_uly*ppi)),
                 Coord(r2i(title_lrx*ppi), r2i(title_lry*ppi))}

    // Set up the map boder style variables.
    solidline = LineStyle({{{1, -1, 0}}})
    black = ColorRGB(0,0,0)

    // Get the map name.
    mapname = GetMap()

    // Create an empty layout to use for the batch printing.
    layout_name = "batchlayout"
    CreateLayout(layout_name,)

    // Loop through the chosen_pts array, create a layout, and print it if not
    // in test mode.
    SetLayer(points_layer)
    if test = 0 then CreateProgressBar(,"True")  // True provides a Cancel button

    for i = 1 to chosen_pts.length do
        if test = 0 then do
            status = UpdateProgressBar("Printing layout " + i2s(i)+
                " of " + i2s(chosen_pts.length)+"...",r2i(i/chosen_pts.length*100))
            if status = "True" then goto quit
            end

        // Get the point's record handle to center the map on the point.
        c = GetPoint(rh2id(chosen_pts[i]))

        // Set the map scope based on the point and the width set in
        // the Administrative Settings, then redraw the map.
        SetMapScope( , Scope(c, map_width_miles, map_width_miles/layout_ar ,0))
        RedrawMap()

        // add map to the layout.
        mapid = AddAnnotation(,"Map",{
	        {"Location",map_loc},
	        {"Map", mapname},
	        {"Scope", GetMapScope(mapname)},
            {"Drop Shadow","False"},
            {"LineWidth", .5},
            {"Line Color", black},
            {"LineStyle", solidline}})

        // Get the name of the map on the layout to use in adding the legend.
        newmapdef = GetAnnotation( , mapid)
        tmp = FindOptionValue(newmapdef[2], "Map")  // get the new map name
        legend_map_name = tmp[1]

        // Add the legend to the layout
        AddAnnotation( ,"Legend",
            // This is the map name on the layout, not the original map name.
  		    {{"Name", legend_map_name}, 
        	{"Location", legend_loc}}) // only need to specify the upper left corner

        // Set the record so you can get the current name to use for title.
        SetRecord(points_layer, chosen_pts[i])

        // Add a title using the name field.
        titleid = AddAnnotation( , "Text",{
            {"Color",black},
            {"Angle",0},
            {"Font","Arial|Bold|18"},
            {"Location", title_loc},
            {"Text", points_layer.(name_field)}})  // use name field for title

        // Redraw the layout.
        DrawLayout()

        // Print the layout, or display a message if in test mode.
        if test = 0 then PrintLayout(1,1)
        else if
            MessageBox("Layout " + i2s(i) + " of " +  i2s(chosen_pts.length),
                {{"Buttons", "YesNo"},
                {"Caption", "Continue?"},
                {"Modal", "Appl"}}) = "No" then goto quit

        // Drop everything on the layout.
        DropAnnotations( , GetAnnotations())
                
        end

    quit:
    if test = 0 then DestroyProgressBar()

    // Close the layout. You could remove this statement to leave the layout open.
    CloseLayout(layout_name)

    // Display completion message to user.
    if test = 0 then ShowMessage("Printed " + i2s(chosen_pts.length) +
        " layouts.  Completed at " + GetDateAndTime())

EndMacro

// Prompt user to choose points and set or clear the test mode.
Dbox "get_choices" (layer, name, test_mode) title: "Choose Points for Batch Mapping"
    // Displays point names in a scroll list; allows user to highlight one or more
    // and copy them to a second scroll list on the right.  
    // Drop, Move Up, and Move Down allow the user to manipulate the liston the right.
    // Returns chosen points record handles an array.

    init do
        // Load the names into the left scroll list.  Include the record handle (rh)
        // in the scroll array but don't display it to the user.  This allows you to
        // keep the record handle with the name and use it later to choose the point
        // in order to zoom to it.

        // Loop through points_layer in alphabetical order and load names and record
        // handles into array.
        SetLayer(layer)
        tot = GetRecordCount(,)
        dim available_list[tot]
        CreateProgressBar(,)
        order = {{name, "Ascending"}}
        rh = GetFirstRecord(layer+"|", order)
        n = 1
        while rh <> null do
            UpdateProgressBar("Loading points...",r2i(n/tot*100))

            // This illustrates a useful trick.
            // Create a formatting array (see Scroll Lists help) to
            // hold the name list, but also include the point's
            // record handle, in a position where the user can"t see it.
            // Later you can extract the chosen record handles to pass back.
            available_list[n] = 
                {{1, "L", layer.name},  // The name for the point
                {35, "R", rh}}          // The points record handle, hidden
                                        // The scroll list is only 30 characters wide
            rh = GetNextRecord(layer+"|",,order)
            n = n+1
            end
        DestroyProgressBar()     

        // The right list starts out empty, so disable the list manipulation buttons.
        DisableItem("Move Up")
        DisableItem("Move Down")
        DisableItem("<<Drop")
        DisableItem("Add>>")

        // Initialize the chosen list to empty (null).
        chosen_list = null

        // Initialize test mode to 1 (checked).
        test_mode = 1
        enditem

    // Left heading and scroll list of available points 
    Text 0, .5, 20 variable: "Available Points"
    Scroll List same, after, 30, 20 multiple list: available_list
        variable: available_list_idxs do

        // Items highlighted, so enable the Add button
        EnableItem("Add>>")
        EndItem

    // Right heading and scroll list of chosen points
    Text 46, .5, 20 variable: "Chosen Points"
    Scroll List "List2" same, after, 30, 20 multiple list: chosen_list
        variable: chosen_list_idxs do

        // Items highlighted, so enable appropriate buttons if possible
        EnableItem("<<Drop")
        RunMacro("fix move buttons")

        EndItem

    // Dialog boxes can have macros to do useful things
    Macro "fix move buttons" do

        // if first highlight is at top, then disable Move Up
        if chosen_list_idxs[1] = 1 then DisableItem("Move Up")
        else EnableItem("Move Up")

        // if last highlight is at bottom, then disable Move Down
        if chosen_list_idxs[chosen_list_idxs.length] = chosen_list.length then 
                DisableItem("Move Down") else EnableItem("Move Down")
        endItem

    // Now do the buttons in between the lists
    Button "Add>>" 32, 2, 12, 1 do
            
        // Copy the points from the avaiable list to the chosen list. You may
        // want to add code here to prevent the user from choosing duplicates.
        for i = 1 to available_list_idxs.length do
            // Add the highlighted items to the current chosen list.
            // This illustrates concatenation of arrays.  You are adding another
            // array element, so it must be enclosed in {}.
            chosen_list = chosen_list + {available_list[available_list_idxs[i]]}
            end

        // Now that the items are copied, clear the highlights and disable the
        // Add button.
        available_list_idxs = null
        chosen_list_idxs = null
        DisableItem("Add>>")
        endItem

    Button "Move Up" same, 5, 12, 1 do
            
        // Loop through all highlighted items
        for i = 1 to chosen_list_idxs.length do

            // New position of element after move, not less than 1
            newpos = R2I(Max(chosen_list_idxs[i]-1, 0))  

            // Swap the elements using a local utility macro
            runmacro("swap_elements", chosen_list, newpos, chosen_list_idxs[i])

            // Change the index to highlight the new position
            chosen_list_idxs[i] = newpos
            end

            // Fix Move Up and Move Down buttons
            RunMacro("fix move buttons")

        endItem

    Button "Move Down" same, 7, 12, 1 do

        // Loop through all highlighted items
        for i = 1 to chosen_list_idxs.length do

            // New position of element after move, not greater than list length
            newpos = R2I(Min(chosen_list_idxs[i]+1, chosen_list.length)) 

            // Swap the elements using a local utility macro
            runmacro("swap_elements", chosen_list, newpos, chosen_list_idxs[i])

            // Change the index to highlight the new position
            chosen_list_idxs[i] = newpos
            end

            // Fix Move Up and Move Down buttons
            RunMacro("fix move buttons")

        EndItem

    Button "<<Drop" same, 10, 12, 1 do

        // Remove each highlighted element from the array.
        // Start from the end of the array so the subscripts don't change.
        for i = chosen_list_idxs.length to 1 step -1 do
            chosen_list = ExcludeArrayElements(chosen_list, chosen_list_idxs[i],1)
            end

        // If the entire list is deleted then disable Drop button.
        if chosen_list = null then DisableItem("<<Drop")

        // Clear the chosen_list indexes because the entries are no longer there.
        chosen_list_idxs = null

        // Disable the right list buttons
        DisableItem("Move Up")
        DisableItem("Move Down")
        DisableItem("<<Drop")

        enditem

    // Checkbox for test mode.
    CheckBox 2, 22.5 prompt: "Test Mode" variable: test_mode

    // The OK and Cancel buttons are at the bottom right.
    Button "OK" 53, same, 10, 1 do
            
        // If nothing is chosen then issue a message.
        if chosen_list.length = 0 then do
            ShowMessage("Please choose at least one point or click Cancel.")
            goto end_ok
            end

        // Create an array of chosen record handles to return to calling macro.
        dim return_array[chosen_list.length]
        for i = 1  to chosen_list.length do
            return_array[i] = chosen_list[i][2][3]
            end
            Return(return_array)

        end_ok:
        endItem

    // This provides a convenient spacer so the next button can use after and you
    // can move the buttons left or right by just changing the OK button.
    Text "   " after, same

    Button "Cancel"  after, same, 10, 1 do
            
        Return(null)  // The calling macro will detect the null and quit
        endItem

EndDbox

Macro "swap_elements" (array, element1, element2)

    // moves element1 of array to element2 and vice versa
    // element1 and element2 are integer array subscripts
    temp = array[element1]
    array[element1] = array[element2]
    array[element2] = temp

EndMacro

